package com.gic.message.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.binlog.base.entity.GicField;
import com.gic.binlog.base.entity.GicRecord;
import com.gic.binlog.base.entity.enums.GicRecordType;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthListDTO;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.message.service.MessageHandler;
import com.gic.message.utils.ListToMapUtil;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreAuthorizationApiService;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.UnionStoreChangeApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
/**
 * 
 * @ClassName: StoreMessageHandler
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 15:12
 */
@Component
public class StoreMessageHandler implements MessageHandler {

    public static final String OLD_STATUS = "old_status";
    public static final String NEW_STATUS = "new_status";
    public static final String NEW_OWN_TYPE = "new_own_type";
    public static final Integer NEW_STATUS_VALUE = 2;
    private static final String OLD_DELETE_FLAG = "old_delete_flag";
    private static final String NEW_DELETE_FALG = "new_delete_flag";

    private static final Logger logger = LogManager.getLogger(StoreMessageHandler.class);

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreIndexRefreshHandler storeIndexRefreshHandler;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    private UnionStoreChangeApiService unionStoreChangeApiService;

    @Override
    public void handler(ConsumerRecord<String, GicRecord> consumerRecord){
        GicRecord value = consumerRecord.value();
        Map<String, GicField> fieldMap = ListToMapUtil.listToMap(consumerRecord);
        Integer enterpriseId = 0, storeId = 0, storeInfoId;
        if(GicRecordType.INSERT.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            this.storeIndexRefreshHandler.refreshStoreIndex(enterpriseId, storeId);
            Integer ownType = Integer.valueOf(fieldMap.get("own_type").getValue());
            this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.INSERT.value());
        }
        if(GicRecordType.UPDATE.value() == value.getRecordType().value()) {
            enterpriseId = Integer.valueOf(fieldMap.get("new_enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("new_store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("new_store_info_id").getValue());
            if (Integer.valueOf(fieldMap.get(OLD_STATUS).getValue()) == 1 && Integer.valueOf(fieldMap.get(NEW_STATUS).getValue()) == 0) {  //删除
                this.storeApiService.deleteStoreFromEs(enterpriseId, storeId);
            }else {
                this.storeIndexRefreshHandler.refreshStoreIndex(enterpriseId, storeId);
            }
            this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.UPDATE.value());
        }
        if(GicRecordType.DELETE.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            this.storeApiService.deleteStoreFromEs(enterpriseId, storeId);
            Integer ownType = Integer.valueOf(fieldMap.get("own_type").getValue());
            this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.DELETE.value());
        }
    }

    private void dealStoreChange(Integer enterpriseId, Integer storeId, Integer storeInfoId, int recordType){
        /****
         * 1 判断当前门店是否在联合商户资源组中 存在 判断联合商户中是否存在  不存在需要进行同步
         * 2 判断当前门店是否在联合商户资源组中 不存在 判断联合商户中是否存在  存在需要进行取消同步
         * ****/
        /**查询商户下的所有联合商户授权资源组**/
        List<UnionEnterpriseAuthDTO> list = this.unionEnterpriseAuthApiService.listUnionEnterpriseAuth(enterpriseId).getResult();
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        for(UnionEnterpriseAuthDTO dto : list){
            /***通过联合商户**/
            ResourceDTO resourceDTO = this.resourceApiService.getResource(dto.getResourceGroupId()).getResult();
            if(resourceDTO == null){
                return;
            }
            StoreWidgetDTO storeWidgetDTO = this.storeWidgetApiService.getStoreWidget(resourceDTO.getStoreResource().intValue()).getResult();
            if(storeWidgetDTO == null){
                return;
            }
            if(AuthModeEnum.YES.getCode().equals(storeWidgetDTO.getAuthMode())){
                StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                storeSearchDTO.setStoreIds(String.valueOf(storeId));
                storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
                storeSearchDTO.setEnterpriseId(enterpriseId);
                Long count = this.storeApiService.queryStoreCountFromEs(storeSearchDTO).getResult();
                boolean ownExist = count > 0 ? true : false;
                Integer unionEnterpriseId  = 1;
                storeSearchDTO.setEnterpriseId(unionEnterpriseId);
                count = this.storeApiService.queryStoreCountFromEs(storeSearchDTO).getResult();
                boolean unionExist = count > 0 ? true : false;
                if(ownExist && !unionExist){
                    StoreDTO storeDTO = new StoreDTO();
                    storeDTO.setStoreInfoId(storeInfoId);
                    storeDTO.setEnterpriseId(enterpriseId);
                    storeDTO.setFromEnterpriseId(dto.getUnionEnterpriseId());
                    this.storeAuthorizationApiService.saveStore(storeDTO);
                    this.unionStoreChangeApiService.save(dto.getOwnEnterpriseId(), dto.getUnionEnterpriseId());
                }
                if(!ownExist && unionExist){
                    this.storeAuthorizationApiService.cancelStore(storeInfoId, dto.getUnionEnterpriseId());
                    this.unionStoreChangeApiService.save(dto.getOwnEnterpriseId(), dto.getUnionEnterpriseId());
                }
            }
        }
    }
}
